@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- $Id: battstat_applet.c,v 1.105 2005/06/19 10:53:56 davyd Exp $
+ $Id: battstat_applet.c,v 1.111 2005/07/14 06:53:16 ryanl Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,7 +55,6 @@
 
 #define GCONF_PATH ""
 
-static void suspend_cb( BonoboUIComponent *, ProgressData *, const char * );
 static void about_cb( BonoboUIComponent *, ProgressData *, const char * );
 static void help_cb( BonoboUIComponent *, ProgressData *, const char * );
 
@@ -63,7 +62,6 @@ static const BonoboUIVerb battstat_menu_verbs [] = {
 	BONOBO_UI_UNSAFE_VERB ("BattstatProperties", prop_cb),
 	BONOBO_UI_UNSAFE_VERB ("BattstatHelp",       help_cb),
 	BONOBO_UI_UNSAFE_VERB ("BattstatAbout",      about_cb),
-	BONOBO_UI_UNSAFE_VERB ("BattstatSuspend",    suspend_cb),
         BONOBO_UI_VERB_END
 };
 
@@ -86,7 +84,8 @@ static const int pixel_offset_bottom[]={ 38, 38, 39, 39, 39, 39, 39, 39, 39, 39,
 
 /* The following array is the colour of each line.  The (slightly) varying
    colours are what makes for the gradient effect.  The 'dark' colours are
-   used to draw the end of the bar, giving it more of a 3D look.
+   used to draw the end of the bar, giving it more of a 3D look.  The code
+   assumes that all of these arrays will have the same number of elements.
 */
 static GdkColor green[] = {
   {0,0x7A00,0xDB00,0x7000},
@@ -101,8 +100,8 @@ static GdkColor green[] = {
   {0,0x6000,0xD000,0x5100},
   {0,0x5600,0xCA00,0x4600},
   {0,0x5100,0xC100,0x4200},
-  {-1,0x0000,0x0000,0x0000}
 };
+
 static GdkColor red[] = {
   {0,0xD900,0x7200,0x7400},
   {0,0xE600,0x8800,0x8C00},
@@ -116,8 +115,8 @@ static GdkColor red[] = {
   {0,0xCD00,0x5400,0x5900},
   {0,0xC800,0x4900,0x4F00},
   {0,0xC100,0x4200,0x4700},
-  {-1,0x0000,0x0000,0x0000}
 };
+
 static GdkColor yellow[] = {
   {0,0xD800,0xD900,0x7200},
   {0,0xE600,0xE500,0x8800},
@@ -131,8 +130,8 @@ static GdkColor yellow[] = {
   {0,0xCD00,0xCC00,0x5400},
   {0,0xC800,0xC600,0x4900},
   {0,0xC100,0xBF00,0x4200},
-  {-1,0x0000,0x0000,0x0000}
 };
+
 static GdkColor orange[] = {
   {0,0xD900,0xAD00,0x7200},
   {0,0xE600,0xBB00,0x8800},
@@ -146,23 +145,8 @@ static GdkColor orange[] = {
   {0,0xCD00,0x9600,0x5400},
   {0,0xC800,0x8D00,0x4900},
   {0,0xC100,0x8600,0x4200},
-  {-1,0x0000,0x0000,0x0000}
-};
-static GdkColor gray[] = {
-  {0,0xC400,0xC400,0xC400},
-  {0,0xD300,0xD300,0xD300},
-  {0,0xDE00,0xDE00,0xDE00},
-  {0,0xD800,0xD800,0xD800},
-  {0,0xD300,0xD300,0xD300},
-  {0,0xCD00,0xCD00,0xCD00},
-  {0,0xC700,0xC700,0xC700},
-  {0,0xC100,0xC100,0xC100},
-  {0,0xBB00,0xBB00,0xBB00},
-  {0,0xB500,0xB500,0xB500},
-  {0,0xAE00,0xAE00,0xAE00},
-  {0,0xA800,0xA800,0xA800},
-  {-1,0x0000,0x0000,0x0000}
 };
+
 static GdkColor darkgreen[] = {
   {0,0x6500,0xC600,0x5B00},
   {0,0x7B00,0xD300,0x6F00},
@@ -176,8 +160,8 @@ static GdkColor darkgreen[] = {
   {0,0x4B00,0xBB00,0x3C00},
   {0,0x4100,0xB600,0x3100},
   {0,0x3C00,0xAC00,0x2D00},
-  {-1,0x0000,0x0000,0x0000}
 };
+
 static GdkColor darkorange[] = {
   {0,0xC400,0x9700,0x5C00},
   {0,0xD000,0xA500,0x7200},
@@ -191,8 +175,8 @@ static GdkColor darkorange[] = {
   {0,0xB800,0x8100,0x3F00},
   {0,0xB300,0x7900,0x3400},
   {0,0xAC00,0x7200,0x2D00},
-  {-1,0x0000,0x0000,0x0000}
 };
+
 static GdkColor darkyellow[] = {
   {0,0xC200,0xC400,0x5C00},
   {0,0xD000,0xCF00,0x7200},
@@ -206,8 +190,8 @@ static GdkColor darkyellow[] = {
   {0,0xB800,0xB700,0x3F00},
   {0,0xB300,0xB200,0x3400},
   {0,0xAC00,0xAA00,0x2D00},
-  {-1,0x0000,0x0000,0x0000}
 };
+
 static GdkColor darkred[] = {
   {0,0xC900,0x6200,0x6400},
   {0,0xD600,0x7800,0x7C00},
@@ -221,7 +205,6 @@ static GdkColor darkred[] = {
   {0,0xB800,0x3F00,0x4400},
   {0,0xB100,0x3200,0x3700},
   {0,0xA200,0x3200,0x3700},
-  {-1,0x0000,0x0000,0x0000}
 };
 
 /* Initialise the global static variables that store our status pixmaps from
@@ -262,36 +245,22 @@ initialise_global_pixmaps( void )
 static void
 allocate_battery_colours( void )
 {
+  GdkColormap *colourmap;
   int i;
 
-  for( i = 0; orange[i].pixel != -1; i++ )
+  colourmap = gdk_colormap_get_system();
+
+  /* assumed: all the colour arrays have the same number of elements */
+  for( i = 0; i < G_N_ELEMENTS( orange ); i++ )
   {
-     gdk_colormap_alloc_color (gdk_colormap_get_system(), 
-			       &darkorange[i], FALSE, TRUE);
-      
-     gdk_colormap_alloc_color (gdk_colormap_get_system(), 
-			       &darkyellow[i], FALSE, TRUE);
-      
-     gdk_colormap_alloc_color (gdk_colormap_get_system (), 
-			       &darkred[i], FALSE, TRUE);
-      
-     gdk_colormap_alloc_color (gdk_colormap_get_system (), 
-			       &darkgreen[i], FALSE, TRUE);
-      
-     gdk_colormap_alloc_color (gdk_colormap_get_system (), 
-			       &orange[i], FALSE, TRUE);
-      
-     gdk_colormap_alloc_color (gdk_colormap_get_system (), 
-			       &yellow[i], FALSE, TRUE);
-      
-     gdk_colormap_alloc_color (gdk_colormap_get_system (), 
-			       &red[i], FALSE, TRUE);
-      
-     gdk_colormap_alloc_color (gdk_colormap_get_system (), 
-			       &green[i], FALSE, TRUE);
-      
-     gdk_colormap_alloc_color (gdk_colormap_get_system (), 
-			       &gray[i], FALSE, TRUE);
+     gdk_colormap_alloc_color( colourmap, &darkorange[i], FALSE, TRUE );
+     gdk_colormap_alloc_color( colourmap, &darkyellow[i], FALSE, TRUE );
+     gdk_colormap_alloc_color( colourmap, &darkred[i], FALSE, TRUE );
+     gdk_colormap_alloc_color( colourmap, &darkgreen[i], FALSE, TRUE );
+     gdk_colormap_alloc_color( colourmap, &orange[i], FALSE, TRUE );
+     gdk_colormap_alloc_color( colourmap, &yellow[i], FALSE, TRUE );
+     gdk_colormap_alloc_color( colourmap, &red[i], FALSE, TRUE );
+     gdk_colormap_alloc_color( colourmap, &green[i], FALSE, TRUE );
   }
 }
 
@@ -365,7 +334,7 @@ get_remaining (BatteryStatus *info)
 	hours = info->minutes / 60;
 	mins = info->minutes % 60;
 
-	if (info->on_ac_power && info->percent == 100)
+	if (info->on_ac_power && !info->charging)
 		return g_strdup_printf (_("Battery charged (%d%%)"), info->percent);
 	else if (info->minutes < 0 && !info->on_ac_power)
 		return g_strdup_printf (_("Unknown time (%d%%) remaining"), info->percent);
@@ -431,6 +400,7 @@ battery_full_notify (void)
 				NULL,		/* body text */
 				icon,		/* icon */
 				TRUE, 0,	/* expiry, server default */
+				NULL,		/* hints */
 				NULL,		/* no user_data */
 				0))		/* no actions */
 		return FALSE;
@@ -714,7 +684,7 @@ update_battery_image (ProgressData *battstat, int batt_percent, int batt_time)
   if (battstat->draintop) {
     progress_value = PROGLEN * batt_life / 100.0;
 	    
-    for (i = 0; color[i].pixel != -1; i++)
+    for( i = 0; i < G_N_ELEMENTS( orange ); i++ )
     {
       gdk_gc_set_foreground (battstat->pixgc, &color[i]);
 
@@ -730,7 +700,7 @@ update_battery_image (ProgressData *battstat, int batt_percent, int batt_time)
   {
     progress_value = PROGLEN * batt_life / 100.0;
 
-    for (i = 0; color[i].pixel != -1; i++)
+    for( i = 0; i < G_N_ELEMENTS( orange ); i++)
     {
       gdk_gc_set_foreground (battstat->pixgc, &color[i]);
 
@@ -743,7 +713,7 @@ update_battery_image (ProgressData *battstat, int batt_percent, int batt_time)
                        pixel_offset_bottom[i] - progress_value);
     }
 
-    for (i = 0; darkcolor[i].pixel != -1; i++)
+    for( i = 0; i < G_N_ELEMENTS( orange ); i++ )
     {
       x = pixel_offset_bottom[i] - progress_value - pixel_top_length[i];
       if (x < pixel_offset_top[i])
@@ -961,13 +931,13 @@ check_for_updates( gpointer data )
       battstat->last_batt_life != 1000 &&
       (
         /* if percentage drops below red_val */
-        !battstat->red_value_is_time &&
-        battstat->last_batt_life > battstat->red_val &&
-        info.percent <= battstat->red_val ||
+        (!battstat->red_value_is_time &&
+         battstat->last_batt_life > battstat->red_val &&
+         info.percent <= battstat->red_val) ||
 	/* if time drops below red_val */
-	battstat->red_value_is_time &&
-	battstat->last_minutes > battstat->red_val &&
-	info.minutes <= battstat->red_val
+	(battstat->red_value_is_time &&
+	 battstat->last_minutes > battstat->red_val &&
+	 info.minutes <= battstat->red_val)
       ) &&
       info.present)
   {
@@ -1012,7 +982,6 @@ check_for_updates( gpointer data )
   if( info.on_ac_power != battstat->last_acline_status ||
       info.percent != battstat->last_batt_life ||
       info.minutes != battstat->last_minutes ||
-      info.state != battstat->last_batt_state ||
       info.charging != battstat->last_charging )
   {
     /* Update the tooltip */
@@ -1043,7 +1012,6 @@ check_for_updates( gpointer data )
   }
 
   battstat->last_charging = info.charging;
-  battstat->last_batt_state = info.state;
   battstat->last_batt_life = info.percent;
   battstat->last_minutes = info.minutes;
   battstat->last_acline_status = info.on_ac_power;
@@ -1071,30 +1039,30 @@ destroy_applet( GtkWidget *widget, ProgressData *battstat )
 
   gtk_timeout_remove( battstat->pixtimer );
 
-  g_object_unref( G_OBJECT(battstat->pixgc) );
+  if( battstat->pixgc )
+    g_object_unref( G_OBJECT(battstat->pixgc) );
+
   g_object_unref( G_OBJECT(battstat->status) );
   g_object_unref( G_OBJECT(battstat->percent) );
   g_object_unref( G_OBJECT(battstat->battery) );
   g_object_unref( G_OBJECT(battstat->tip) );
 
-  if( battstat->suspend_cmd )
-    g_free( battstat->suspend_cmd );
-   
   g_free( battstat );
 
   static_global_teardown();
 }
 
-/* Called when the user selects the 'help' menu item.
+/* Common function invoked by the 'Help' context menu item and the 'Help'
+ * button in the preferences dialog.
  */
-static void
-help_cb( BonoboUIComponent *uic, ProgressData *battstat, const char *verb )
+void
+battstat_show_help( ProgressData *battstat, const char *section )
 {
   GError *error = NULL;
 
-  gnome_help_display_on_screen( "battstat", NULL,
-		                gtk_widget_get_screen( battstat->applet ),
-		                &error );
+  gnome_help_display_on_screen( "battstat", section,
+                                gtk_widget_get_screen( battstat->applet ),
+                                &error );
 
   if( error )
   {
@@ -1108,44 +1076,13 @@ help_cb( BonoboUIComponent *uic, ProgressData *battstat, const char *verb )
   }
 }
 
-/* Called when the user chooses the 'suspend' menu item or double-clicks.
+
+/* Called when the user selects the 'help' menu item.
  */
 static void
-suspend_cb (BonoboUIComponent *uic,
-	    ProgressData      *battstat,
-	    const char        *verb)
+help_cb( BonoboUIComponent *uic, ProgressData *battstat, const char *verb )
 {
-   if(battstat->suspend_cmd && strlen(battstat->suspend_cmd)>0) {
-      GError *err = NULL;
-      gboolean ret;
-      gint shell_ret = 0;
-
-      ret = g_spawn_command_line_sync(battstat->suspend_cmd,
-		      NULL, NULL, &shell_ret, &err);
-      if (ret == FALSE || shell_ret != 0)
-      {
-	      gchar *msg;
-
-	      if (err != NULL)
-	      {
-		      msg = g_strdup_printf(_("An error occured while launching the Suspend command: %s\nPlease try to correct this error"), err->message);
-	      } else {
-		      /* Probably because the shell_ret is != 0 */
-		      if (battstat->suspend_cmd)
-		          msg = g_strdup_printf(_("The Suspend command '%s' was unsuccessful."), battstat->suspend_cmd);
-		      else
-		          msg = g_strdup_printf(_("The Suspend command was unsuccessful."));
-	      }
-	      battstat_error_dialog (battstat->applet, msg);
-	      g_free(msg);
-	      if (err != NULL)
-		      g_error_free(err);
-      }
-   } else {
-      battstat_error_dialog (battstat->applet, _("Suspend command wasn't setup correctly in the preferences.\nPlease change the preferences and try again."));
-   }
-   
-   return;
+  battstat_show_help( battstat, NULL );
 }
 
 /* Called when the user selects the 'about' menu item.
@@ -1275,39 +1212,6 @@ change_background (PanelApplet *a,
 	}
 }
 
-/* Suspend if we're double-clicked.
- */
-static gboolean
-button_press_cb (GtkWidget *widget, GdkEventButton *event, ProgressData *battstat)
-{
-	if (event->button == 1 && event->type == GDK_2BUTTON_PRESS) 
-		suspend_cb (NULL, battstat, NULL);
-
-	return FALSE;
-}
-
-/* Suspend for the listed keys.
- */
-static gboolean
-key_press_cb (GtkWidget *widget, GdkEventKey *event, ProgressData *battstat)
-{
-	switch (event->keyval) {
-	case GDK_KP_Enter:
-	case GDK_ISO_Enter:
-	case GDK_3270_Enter:
-	case GDK_Return:
-	case GDK_space:
-	case GDK_KP_Space:
-		suspend_cb (NULL, battstat, NULL);
-		return TRUE;
-
-	default:
-		break;
-	}
-
-	return FALSE;
-}
-
 /* Get our settings out of gconf.
  */
 static void
@@ -1343,8 +1247,6 @@ load_preferences(ProgressData *battstat)
 	  battstat->showbattery = FALSE;
   
   battstat->showtext = panel_applet_gconf_get_int (applet, GCONF_PATH "show_text", NULL);
-  battstat->suspend_cmd = panel_applet_gconf_get_string (applet, GCONF_PATH "suspend_command", NULL);
-  
 }
 
 /* Convenience function to attach a child widget to a GtkTable in the
@@ -1600,16 +1502,6 @@ create_layout(ProgressData *battstat)
 		    G_CALLBACK (change_background),
 		    battstat);
 
-  g_signal_connect (battstat->applet,
-		    "button_press_event",
-   		    G_CALLBACK (button_press_cb),
-		    battstat);
-
-  g_signal_connect (battstat->applet,
-		    "key_press_event",
-		    G_CALLBACK (key_press_cb),
-		    battstat);
-
   return FALSE;
 }
 
@@ -1640,10 +1532,8 @@ battstat_applet_fill (PanelApplet *applet)
   battstat->refresh_label = TRUE;
   battstat->last_batt_life = 1000;
   battstat->last_acline_status = 1000;
-  battstat->last_batt_state = 1000;
   battstat->last_pixmap_index = 1000;
   battstat->last_charging = 1000;
-  battstat->suspend_cmd = NULL;
   battstat->orienttype = panel_applet_get_orient (applet);
   battstat->horizont = TRUE;
   battstat->battery_low_dialog = NULL;
