@@ -19,12 +19,14 @@
 #include "gswitchit-applet.h"
 #include "libgswitchit/gswitchit_util.h"
 #include "libgswitchit/gswitchit_plugin_manager.h"
+#include "libkbdraw/keyboard-drawing.h"
 
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <stdlib.h>
 
 #include <gdk/gdkscreen.h>
 #include <gdk/gdkx.h>
@@ -34,6 +36,20 @@
 
 #define GROUPS_SUBMENU_PATH "/popups/popup/groups"
 
+/* we never build applet without XKB, so far: TODO */
+#define HAVE_XKB  1
+
+#ifdef HAVE_XKB
+#include "X11/XKBlib.h"
+/**
+ * BAD STYLE: Taken from xklavier_private_xkb.h
+ * Any ideas on architectural improvements are WELCOME
+ */
+extern Bool _XklXkbConfigPrepareNative( const XklConfigRecPtr data, XkbComponentNamesPtr componentNamesPtr );
+extern void _XklXkbConfigCleanupNative( XkbComponentNamesPtr componentNamesPtr );
+/* */
+#endif
+
 /* one instance for ALL applets */
 static GSwitchItAppletGlobals globals;
 
@@ -56,6 +72,10 @@ static void GSwitchItAppletCmdCapplet (BonoboUIComponent * uic,
 				       GSwitchItApplet * sia,
 				       const gchar * verb);
 
+static void GSwitchItAppletCmdPreview (BonoboUIComponent * uic,
+				       GSwitchItApplet * sia,
+				       const gchar * verb);
+
 static void GSwitchItAppletCmdPlugins (BonoboUIComponent * uic,
 				       GSwitchItApplet * sia,
 				       const gchar * verb);
@@ -91,6 +111,7 @@ static gboolean GSwitchItAppletKeyPressed (GtkWidget * widget,
 
 static const BonoboUIVerb gswitchitAppletMenuVerbs[] = {
 	BONOBO_UI_UNSAFE_VERB ("Capplet", GSwitchItAppletCmdCapplet),
+	BONOBO_UI_UNSAFE_VERB ("Preview", GSwitchItAppletCmdPreview),
 	BONOBO_UI_UNSAFE_VERB ("Plugins", GSwitchItAppletCmdPlugins),
 	BONOBO_UI_UNSAFE_VERB ("About", GSwitchItAppletCmdAbout),
 	BONOBO_UI_UNSAFE_VERB ("Help", GSwitchItAppletCmdHelp),
@@ -227,7 +248,7 @@ GSwitchItAppletFilterXEvt (GdkXEvent * xev, GdkEvent * event)
 			XReparentEvent *rne = (XReparentEvent *) xev;
 
 			ForAllApplets ()
-				GdkWindow * w = sia->appletAncestor->window;
+				GdkWindow * w = sia->appletAncestor;
 				/* compare the applet's parent window with the even window */
 				if (w != NULL && GDK_WINDOW_XID (w) == rne->window) {
 					/* if so - make it transparent...*/
@@ -306,10 +327,13 @@ GSwitchItAppletPrepareDrawing (GSwitchItApplet * sia, int group)
 		gtk_container_add (GTK_CONTAINER (sia->ebox), flagImg);
 		g_object_unref (G_OBJECT (scaled));
 	} else {
-		char *layoutName;
-		char *allocLayoutName = NULL;
+		char *layoutName = NULL;
 		XklConfigItem configItem;
 		GtkWidget *align, *label;
+		static GHashTable *shortDescrs = NULL;
+		if (group == 0)
+			shortDescrs = g_hash_table_new_full (g_str_hash, g_str_equal,
+							     g_free, NULL);
 
 		if (XklGetBackendFeatures() & XKLF_MULTIPLE_LAYOUTS_SUPPORTED) {
 			char *fullLayoutName =
@@ -319,7 +343,7 @@ GSwitchItAppletPrepareDrawing (GSwitchItApplet * sia, int group)
 			if (!GSwitchItKbdConfigSplitItems
 			    (fullLayoutName, &layoutName, &variantName))
 				/* just in case */
-				layoutName = fullLayoutName;
+				layoutName = g_strdup (fullLayoutName);
 
 			g_snprintf (configItem.name,
 				    sizeof (configItem.name), "%s",
@@ -328,17 +352,49 @@ GSwitchItAppletPrepareDrawing (GSwitchItApplet * sia, int group)
 			if (XklConfigFindLayout (&configItem)) {
 				char *sd = configItem.shortDescription;
 				if (sd != NULL && *sd != '\0') {
-					layoutName = allocLayoutName =
+					layoutName = 
 					    g_locale_to_utf8 (sd, -1, NULL,
 							      NULL, NULL);
 				}
 			}
 		} else
-			layoutName = g_slist_nth_data (globals.groupNames, group);
+			layoutName = g_strdup (g_slist_nth_data (globals.groupNames, group));
+
+		if (layoutName == NULL)
+			layoutName = g_strdup ("?");
+
+		gpointer pcounter = NULL; 
+		char *prevLayoutName = NULL;
+		char *labelTitle = NULL;
+		int counter = 0;
+		if (g_hash_table_lookup_extended (shortDescrs, layoutName, 
+						  (gpointer*)&prevLayoutName, &pcounter))
+		{
+			/* "next" same description */
+			char* appendix = NULL;
+			counter = GPOINTER_TO_INT(pcounter);
+			appendix = g_strnfill (counter, '*');
+			labelTitle = g_strconcat (layoutName, appendix, NULL);
+			g_free (appendix);
+			/* layoutName is g_freed inside g_hash_table_insert function */
+		}
+		else
+		{
+			/* "first" time this description */
+			labelTitle = g_strdup (layoutName);
+		}
+		g_hash_table_insert (shortDescrs, layoutName, GINT_TO_POINTER (counter + 1));
+
 		align = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
-		label = gtk_label_new (layoutName);
-		if (allocLayoutName != NULL)
-			g_free (allocLayoutName);
+		label = gtk_label_new (labelTitle);
+		g_free (labelTitle);
+
+		if (group == XklGetNumGroups ())
+		{
+			g_hash_table_destroy (shortDescrs);
+			shortDescrs = NULL;
+		}
+
 		gtk_container_add (GTK_CONTAINER (align), label);
 		gtk_container_add (GTK_CONTAINER (sia->ebox), align);
 
@@ -515,6 +571,112 @@ GSwitchItAppletCmdCapplet (BonoboUIComponent *
 	}
 }
 
+static void 
+GSwitchItPreviewResponse (GtkWidget *dialog, gint resp)
+{
+	switch (resp) 
+	{
+		case GTK_RESPONSE_CLOSE:
+			gtk_widget_destroy (dialog);
+		default:;
+	}
+}
+
+static void 
+GSwitchItPreviewDestroy (GtkWidget *dialog, gint group)
+{
+	GladeXML *gladeData = GLADE_XML (gtk_object_get_data (GTK_OBJECT (dialog), "gladeData"));
+	g_object_unref (G_OBJECT (gladeData));
+	g_hash_table_remove (globals.previewDialogs, GINT_TO_POINTER (group));
+}
+
+void
+GSwitchItAppletCmdPreview (BonoboUIComponent *
+			   uic, GSwitchItApplet * sia, const gchar * verb)
+{
+	static KeyboardDrawingGroupLevel groupsLevels[] = { {0,1}, {0,3}, {0,0}, {0,2} };
+	static KeyboardDrawingGroupLevel * pGroupsLevels[] = {
+		groupsLevels, groupsLevels + 1, groupsLevels + 2, groupsLevels + 3 };
+	XklState *xklState = XklGetCurrentState ();
+	gpointer p = g_hash_table_lookup (globals.previewDialogs, GINT_TO_POINTER (xklState->group));
+	if (p != NULL)
+	{
+		/* existing window */
+		gtk_window_present (GTK_WINDOW (p));
+		return;
+	}
+#ifdef HAVE_XKB
+	GladeXML *gladeData = glade_xml_new (GNOME_GLADEDIR "/gswitchit.glade", "gswitchit_layout_view", NULL);
+        GtkWidget *dialog =
+        	glade_xml_get_widget (gladeData, "gswitchit_layout_view");
+	GtkWidget *kbdraw = keyboard_drawing_new ();
+	XkbComponentNamesRec componentNames;
+
+	if (xklState->group >= 0 && 
+	    xklState->group < g_slist_length (globals.groupNames))
+	{
+		char title[128];
+		snprintf (title, sizeof (title),
+			  _("Keyboard Layout \"%s\""), 
+			  g_slist_nth_data (globals.groupNames, xklState->group));
+		gtk_window_set_title (GTK_WINDOW(dialog), title);
+	}
+
+  	keyboard_drawing_set_groups_levels (KEYBOARD_DRAWING (kbdraw), pGroupsLevels);
+
+	XklConfigRec xklData;
+	XklConfigRecInit (&xklData);
+      	if (XklConfigGetFromServer (&xklData))
+	{
+		if (xklState->group >= 0 && 
+		    xklState->group < xklData.numLayouts &&
+		    xklState->group < xklData.numVariants)
+		{
+			char * l = g_strdup (xklData.layouts[xklState->group]);
+			char * v = g_strdup (xklData.variants[xklState->group]);
+			char ** p;
+			int i;
+
+			if( ( p = xklData.layouts ) != NULL )
+				for( i = xklData.numLayouts; --i >= 0; )
+					free( *p++ );
+
+			if( ( p = xklData.variants ) != NULL )
+				for( i = xklData.numVariants; --i >= 0; )
+					free( *p++ );
+
+			xklData.numLayouts = xklData.numVariants = 1;
+			xklData.layouts  = realloc (xklData.layouts, sizeof (char*));
+			xklData.variants = realloc (xklData.variants, sizeof (char*));
+			xklData.layouts[0] = l;
+			xklData.variants[0] = v;
+		}
+
+		if (_XklXkbConfigPrepareNative (&xklData, &componentNames))
+		{
+			keyboard_drawing_set_keyboard (KEYBOARD_DRAWING (kbdraw), &componentNames);
+			_XklXkbConfigCleanupNative (&componentNames);
+		}
+	}
+	XklConfigRecDestroy (&xklData);
+
+	gtk_object_set_data (GTK_OBJECT (dialog), "gladeData", gladeData);
+	g_signal_connect (GTK_OBJECT (dialog),
+			  "destroy", G_CALLBACK (GSwitchItPreviewDestroy),
+			  GINT_TO_POINTER (xklState->group));
+	g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK(GSwitchItPreviewResponse), NULL);
+
+	gtk_window_resize (GTK_WINDOW (dialog), 700, 400);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
+
+	gtk_container_add (GTK_CONTAINER (glade_xml_get_widget (gladeData, "preview_vbox")), kbdraw);
+	
+	g_hash_table_insert (globals.previewDialogs, GINT_TO_POINTER (xklState->group), dialog);
+
+	gtk_widget_show_all (GTK_WIDGET (dialog));
+#endif
+}
+
 void
 GSwitchItAppletCmdPlugins (BonoboUIComponent *
 			   uic, GSwitchItApplet * sia, const gchar * verb)
@@ -734,8 +896,7 @@ GSwitchItAppletInit (GSwitchItApplet * sia, PanelApplet * applet)
 	gtk_widget_show_all (sia->applet);
 	gtk_widget_realize (sia->applet);
 
-	sia->appletAncestor = gtk_widget_get_ancestor (sia->applet,
-						       GTK_TYPE_WINDOW);
+	sia->appletAncestor = gtk_widget_get_parent_window (sia->applet);
 
 	if (GSwitchItAppletFirstInstance ()) {
 		/* GSwitchItInstallGlibLogAppender(  ); */
@@ -779,6 +940,7 @@ GSwitchItAppletInit (GSwitchItApplet * sia, PanelApplet * applet)
 		GSwitchItAppletConfigActivate (&globals.appletConfig);
 
 		globals.groupNames = GSwitchItConfigLoadGroupDescriptionsUtf8 (&globals.config);
+		globals.previewDialogs = g_hash_table_new (g_direct_hash, g_direct_equal);
 
 		GSwitchItPluginManagerInit (&globals.pluginManager);
 		GSwitchItPluginManagerInitEnabledPlugins (&globals.pluginManager,
@@ -835,6 +997,8 @@ GSwitchItAppletTerm (PanelApplet * applet, GSwitchItApplet * sia)
 		GSwitchItPluginManagerTermInitializedPlugins (&globals.pluginManager);
 		GSwitchItPluginManagerTerm (&globals.pluginManager);
 
+		g_hash_table_destroy (globals.previewDialogs);
+
 		GSwitchItAppletConfigTerm (&globals.appletConfig);
 		GSwitchItKbdConfigTerm (&globals.kbdConfig);
 		GSwitchItConfigTerm (&globals.config);
