@@ -41,8 +41,8 @@ void popup_properties_cb(GtkWidget *widget, StickyNote *note);
 void properties_apply_title_cb(StickyNote *note);
 void properties_apply_color_cb(StickyNote *note);
 void properties_apply_font_cb(StickyNote *note);
-void properties_color_cb(GnomeColorPicker *cp, guint r, guint g, guint b, guint a, StickyNote *note);
-void properties_font_cb(GnomeFontPicker *fp, gchar *font_str, StickyNote *note);
+void properties_color_cb(GtkWidget *button, StickyNote *note);
+void properties_font_cb(GtkWidget *button, StickyNote *note);
 void properties_activate_cb(GtkWidget *widget, StickyNote *note);
 
 #endif /* __STICKYNOTES_CALLBACKS_H__ */
